import os
import json
import requests
import uuid
import getpass
from endstone.plugin import Plugin
from endstone import Player, ColorFormat
from endstone.command import Command, CommandSender
from endstone.event import event_handler, PlayerJoinEvent
import ipaddress


class MyPlugin(Plugin):
    api_version = '0.6'

    def on_enable(self):
        # 创建插件目录
        self.plugin_dir = os.path.join(os.getcwd(), 'plugins', 'CloudBan')
        if not os.path.exists(self.plugin_dir):
            os.makedirs(self.plugin_dir)

        # 配置文件路径
        self.config_path = os.path.join(self.plugin_dir, 'config.json')
        self.players_path = os.path.join(self.plugin_dir, 'players.json')

        # 加载或创建配置
        if not os.path.exists(self.config_path):
            # 创建新配置
            config_data = {
                'token': self.generate_token(32),
                'api_url': 'http://uniteban.xyz:19132/api.php',
                'update_url': 'http://uniteban.xyz:19132/innerapi.php'
            }
            with open(self.config_path, 'w') as f:
                json.dump(config_data, f, indent=4)
            self._config = config_data  # 使用 _config 而不是 config
        else:
            # 加载现有配置
            with open(self.config_path, 'r') as f:
                self._config = json.load(f)  # 使用 _config 而不是 config

        # 加载玩家数据
        if os.path.exists(self.players_path):
            with open(self.players_path, 'r') as f:
                self.players = json.load(f)
        else:
            self.players = {}

        self.logger.info(f'{ColorFormat.YELLOW}CloudBan 已启用...')
        self.register_events(self)

        self.register_server()

    def register_server(self):
        """在插件启动时向云端注册服务器，只发送mac和token"""
        try:
            server_mac = self.get_server_mac()
            token = self._config['token']

            post_data = {
                'mac': server_mac,
                'token': token
            }

            self.logger.info(f'正在向云端注册服务器，MAC: {server_mac}')
            response = requests.post(
                self._config['update_url'],
                json=post_data,
                headers={'Content-Type': 'application/json'},
                timeout=5
            )

            if response.status_code == 200:
                result = response.json()
                if result.get('status') == 'success':
                    self.logger.info('服务器注册成功')
                #else:
                    #self.logger.warning(f'服务器注册失败: {result.get("message", "未知错误")}')
            #else:
                #self.logger.error(f'服务器注册API请求失败: {response.status_code}')
        except Exception as e:
            self.logger.error(f'服务器注册请求异常: {str(e)}')


    @property
    def config(self):
        """提供只读访问配置"""
        return self._config

    def generate_token(self, length):
        """生成指定长度的随机token"""
        return uuid.uuid4().hex[:length]

    def save_config(self):
        """保存配置"""
        with open(self.config_path, 'w') as f:
            json.dump(self._config, f, indent=4)

    def save_players(self):
        """保存玩家数据"""
        with open(self.players_path, 'w') as f:
            json.dump(self.players, f, indent=4)

    def get_server_mac(self):
        """获取服务器MAC地址"""
        try:
            mac = ':'.join(['{:02x}'.format((uuid.getnode() >> elements) & 0xff)
                            for elements in range(0, 8 * 6, 8)][::-1])
            return mac
        except:
            return "00:00:00:00:00:00"

    def check_cloud_ban(self, player):
        """检查玩家是否被云端封禁"""
        player_data = {
            'name': player.name,
            'xuid': player.xuid,
            'uuid': str(player.unique_id),
            'clientid': player.device_id if hasattr(player, 'device_id') else None,
            'ip': player.address.hostname
        }

        # 过滤空值
        post_data = {k: v for k, v in player_data.items() if v is not None}

        try:
            response = requests.post(
                self._config['api_url'],  # 使用 _config
                json=post_data,
                headers={'Content-Type': 'application/json'},
                timeout=5
            )

            if response.status_code == 200:
                result = response.json()
                if result.get('exists', False):
                    self.logger.warning(f'玩家 {player.name} 联合封禁-检查不通过')

                    # 获取服务器MAC地址并更新信息
                    server_mac = self.get_server_mac()
                    self.update_ban_info(
                        player.name,
                        player.xuid,
                        str(player.unique_id),
                        player.device_id if hasattr(player, 'device_id') else None,
                        player.address.hostname,
                        server_mac
                    )

                    # 踢出玩家
                    player.kick('联合封禁-检查不通过')
                    return True
            else:
                self.logger.error(f'封禁检查API请求失败: {response.status_code}')
        except Exception as e:
            self.logger.error(f'封禁检查请求异常: {str(e)}')
        return False

    def update_ban_info(self, name, xuid, uuid, clientid, ip, mac):
        """更新封禁信息到云端"""
        # 构造 post_data
        post_data = {
            'name': name,
            'xuid': xuid,
            'uuid': uuid,
            'clientid': clientid,
            'mac': mac,
            'token': self._config['token']  # 使用 _config
        }

        if ip and not is_private_ip(ip):
            post_data['ip'] = ip

        # 过滤空值
        post_data = {k: v for k, v in post_data.items() if v is not None}

        try:
            response = requests.post(
                self._config['update_url'],  # 使用 _config
                json=post_data,
                headers={'Content-Type': 'application/json'},
                timeout=5
            )

            if response.status_code == 200:
                result = response.json()
                if result.get('status') == 'success':
                    self.logger.info(f'玩家 {name} 封禁数据已上传至云端')
                else:
                    self.logger.warning(f'封禁数据上传失败: {result.get("message", "未知错误")}')
            else:
                self.logger.error(f'封禁更新API请求失败: {response.status_code}')
                self.logger.error("请前往网站 http://uniteban.xyz:19132/ 申请权限")
        except Exception as e:
            self.logger.error(f'封禁更新请求异常: {str(e)}')

    @event_handler
    def on_player_join(self, event: PlayerJoinEvent):
        player = event.player
        player_name = player.name

        # 保存玩家信息
        self.players[player_name] = {
            'uuid': str(player.unique_id),
            'xuid': player.xuid,
            'ip': player.address.hostname,
            'device_id': player.device_id if hasattr(player, 'device_id') else None
        }
        self.save_players()

        # 检查云端封禁
        self.check_cloud_ban(player)

    commands = {
        'cloudban': {
            'description': '封禁玩家到云端',
            'usages': ['/cloudban <player: string>'],  # 使用英文参数名避免解析问题
            'permissions': ['cloudban.command.cloudban']
        }
    }

    permissions = {
        'cloudban.command.cloudban': {
            'description': '封禁玩家到云端',
            'default': 'op'
        }
    }

    def on_command(self, sender: CommandSender, command: Command, args: list[str]):
        if command.name == 'cloudban':
            # if not isinstance(sender, Player):
            #     sender.send_message(f'{ColorFormat.RED}该命令只能由玩家执行')
            #     return

            if isinstance(sender, Player) and not sender.is_op:
                sender.send_message(f'{ColorFormat.RED}你没有权限执行此命令')
                return

            if len(args) < 1:
                sender.send_message(f'{ColorFormat.RED}用法: /cloudban <player>')
                return

            player_name = args[0]

            # 查找玩家信息
            player_info = self.players.get(player_name)
            if not player_info:
                sender.send_message(f'{ColorFormat.RED}找不到玩家 {player_name} 的信息')
                return

            # 获取服务器MAC地址
            server_mac = self.get_server_mac()

            # 更新封禁信息到云端
            self.update_ban_info(
                player_name,
                player_info['xuid'],
                player_info['uuid'],
                player_info.get('device_id'),
                player_info['ip'],
                server_mac
            )

            sender.send_message(f'{ColorFormat.GREEN}玩家 {player_name} 的封禁信息已上传到云端')

def is_private_ip(ip_str):
    try:
        ip = ipaddress.ip_address(ip_str)
        return ip.is_private
    except ValueError:
        # 如果不是合法的IP地址，可以选择认为是私有或忽略
        return False  # 或者根据需求设为 True 或抛异常